# -*- coding: utf-8 -*-
"""
    GESTIKK
    Version 0.6.1

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007-2008:
    Friedrich 'fred.reichbier' Weber, Jonas 'Dauerbaustelle' Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""

def check_version(ver, ref=(2, 12, 0)):
    """ check whether version `ver` is greater than `ref` """
    if ver == ref:
        return True
    for index, vi in enumerate(ver):
        if vi < ref[index]:
            return False
    return True

def _translate_modifier(old):
    """ convert gestikk-modifier notation to gnome """
    return {'ctrl':'<Control>',
            'control':'<Control>',
            'alt':'<Alt>',
            'shift':'<Shift>',
            'super':'<Super>'}[old.lower()]

def convert_keystroke(old):
    """ convert keystrokes from "ctrl+alt+a" to "<Control><Alt>a" """
    if old[-1] == '+':
        splitted = old.split('+')[:-2] + ['+']
    else:
        splitted = old.split('+')
    # convert
    new = []
    for modifier in splitted[:-1]:
        new.append(_translate_modifier(modifier))
    new.append(splitted[-1])
    return ''.join(new)

def migrate_051_to_06(logic):
    """ migrate config and gestures from 0.5.1 to 0.6 """
    for actions in logic.gestures.values():
        for action in actions:
            if action.__class__.__name__ == 'Action_Key': # TODO: working?
                # migrate keystroke
                action.parameter = convert_keystroke(action.parameter)
    return True

MIGRATE_FUNCS = {(0, 5, 1): (migrate_051_to_06, (0, 6))} # old version: (func, new version)

def migrate(logic, old_version, new_version):
    """ check whether we have to migrate.
        :param old_version: tupel 0.5.1 -> (0, 5, 1)
        :param new_version: the current version 0.6 -> (0, 6)
    """
    if check_version(new_version, old_version):
        if old_version in MIGRATE_FUNCS:
            old_human = '.'.join([str(x) for x in old_version])
            new_human = '.'.join([str(x) for x in new_version])
            print 'Going to migrate from version %s to version %s ...' % (old_human, new_human)
            MIGRATE_FUNCS[old_version][0](logic)
            old_version = MIGRATE_FUNCS[old_version][1]
            if old_version != new_version:
                migrate(logic, old_version, new_version)
        else:
            return False
    else:
        return False # we don't have to migrate

