# -*- coding: utf-8 -*-
"""
    GESTIKK
    Version 0.6.1

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007-2008:
    Friedrich 'fred.reichbier' Weber, Jonas 'Dauerbaustelle' Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

"""
    don't use wnck, don't use netk, WE USE NETWM!
    http://standards.freedesktop.org/wm-spec/wm-spec-latest.html
"""

import gtk

class DesktopException(Exception):
    pass

def get_process_by_pid(pid):
    """ get a process name by its pid """
    try:
        f = file('/proc/%d/status' % pid, 'r')
    except IOError:
        return '<could not be recognized>'
    name = '<could not be recognized>'
    while 1:
        line = f.readline().lower()
        if line.startswith('name'):
            name = line[len('name:'):].strip()
            break
    return name

class NETWMHandler():
    def __init__(self):
        self.root = gtk.gdk.get_default_root_window()
        try:
            supported = self.root.property_get('_NET_SUPPORTED')[2]
            for prop in ['_NET_ACTIVE_WINDOW', '_NET_CURRENT_DESKTOP']:
                if prop not in supported:
                    print 'Your window manager does not support netwm feature "%s"!' % prop
                    print "That could, but doesn't have to be a reason that gestikk does not work. If gestikk does not work, please report a bug."
        except TypeError:
            raise DesktopException('Your window manager does not support netwm. Conditions will be disabled.')

    def get_all(self):
        """ get a dict with all values """
        return {'activeApplication':self.get_active_app_name(), \
                'activeWorkspace':self.get_active_workspace_number(), \
                'activeWindowTitle':self.get_active_window_title()}

    def _get_active_window(self):
        """ return current window as gtk.gdk.Window() """
        try:
            return gtk.gdk.window_foreign_new(self.root.property_get('_NET_ACTIVE_WINDOW')[2][0])
        except Exception, e:
            print 'ERROR with netwm. Please report a bug.'
            return None

    def _get_window_name(self, window):
        """ return the window name of the gtk.gdk.Window `window` """
        if not window:
            return
        try:
            return window.property_get('_NET_WM_NAME')[2]
        except TypeError:
            try:
                return window.property_get('WM_NAME')[2]
            except TypeError:
                return window.property_get('WM_CLASS').split('\x00')[1].strip()

    def _get_window_pid(self, window):
        """ return the process id of the current window """
        if not window:
            return
        return window.property_get('_NET_WM_PID')[2][0]

    def get_active_window_title(self):
        """ return the active window title as str """
        return self._get_window_name(self._get_active_window())

    def get_active_app_name(self):
        """ return the active application name as str """
        pid = self._get_window_pid(self._get_active_window())
        if pid:
            return get_process_by_pid(pid)
        else: # use ICCCM
            cmd = self._get_active_window().property_get('WM_CLASS')
            if cmd:
                return cmd[2].split('\x00')[0].strip()

    def get_active_workspace_number(self):
        """ return active workspace number starting at 1 """
        try:
            return self.root.property_get('_NET_CURRENT_DESKTOP')[2][0] + 1
        except IndexError: # does not support desktops (e.g. matchbox)
            return 1 # return 1 (the first)
        
class Dummy():
    def __init__(self):
        pass

    def __getattr__(self):
        return ''

    def get_all(self):
        return {}

if __name__ == '__main__':
    handler = NETWMHandler()
    for k, v in handler.get_all().iteritems():
        print k, ':', v
