# -*- coding: iso-8859-1 -*-
"""
    GESTIKK
    Version 0.6.1

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007-2008:
    Friedrich 'fred.reichbier' Weber (2007-10-08) and
    Jonas 'Dauerbaustelle' Haag (2008)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""

import gtk
import pango

class Window():
    def __init__(self, color='red', line_width=5):
        """
            :param color: The foreground color we want to use. can be named ('black', 'red', etc.) or hex ('#ff0000', '#abcdef')
            :param line_width: The width of a line
        """
        # create the window (without decorations and display in the taskbar through override_redirect)
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # get screen
        self.screen = self.window.get_screen()
        # get size
        self.size = (self.screen.get_width(), self.screen.get_height())
        # get the colormap and the fg color
        self.colormap = self.screen.get_rgb_colormap()
        self.color = self.colormap.alloc_color(color)
        self.c_black = self.colormap.alloc_color('black')
        self.c_white = self.colormap.alloc_color('white')
        self.window.set_default_size(self.size[0], self.size[1])
        self.window.set_decorated(False)
        self.window.set_colormap(self.colormap)
        self.window.show()
        self.window.window.set_override_redirect(True)
        # create the pixmap
        self.pixmap = gtk.gdk.Pixmap(self.window.window, self.size[0], self.size[1])
        self.pixmap.set_colormap(self.colormap)
        self.pixmap.gc = self.pixmap.new_gc(foreground=self.color, line_width=line_width)
        self.pixmap.gc.set_colormap(self.colormap)
        # create the mask
        self.mask = gtk.gdk.Pixmap(None, self.size[0], self.size[1], 1)
        self.mask.gc = self.mask.new_gc(foreground=self.c_white, line_width=line_width)
        # reset everything
        self.clear()
        self.window.stick()
        # set `old_pos`
        self.old_pos = None

    def destroy(self):
        """ destroy the window """
        self.window.destroy()

    def paint_gesture_path(self, x, y):
        """ paint a new station in the gesture's path """
        if self.old_pos:
            self.draw_line(self.old_pos[0], self.old_pos[1], x, y)
        self.old_pos = (x, y)

    def clear(self):
        """ clear + remove all """
        self.window.hide()
        self.reset_mask()
        self.reset_pixmap()
        self.update_mask(False)
        self.update_pixmap()
        self.window.hide()
        self.old_pos = None

    def draw_line(self, x1, y1, x2, y2):
        """ wrapper for a line """
        # draw on pixmap
        self.pixmap.draw_line(self.pixmap.gc, x1, y1, x2, y2)
        # draw on mask
        self.mask.draw_line(self.mask.gc, x1, y1, x2, y2)
        # update all
        self.update_mask()
        self.update_pixmap()

    def reset_mask(self):
        """ reset the shape mask to fully transparent """
        # fill black
        self.mask.gc.set_foreground(self.c_black) #todo?
        self.mask.draw_rectangle(self.mask.gc, True, 0, 0, self.size[0], self.size[1])
        self.mask.gc.set_foreground(self.c_white)

    def reset_pixmap(self):
        """ reset the pixmap to fully black """
        # fill black
        self.pixmap.gc.set_foreground(self.c_black) #todo?
        self.pixmap.draw_rectangle(self.pixmap.gc, True, 0, 0, self.size[0], self.size[1])
        self.pixmap.gc.set_foreground(self.color)

    def update_mask(self, do_show=True):
        """ update the mask """
        self.window.window.input_shape_combine_mask(self.mask, 0, 0) # vllt nach .show()?
        self.window.window.shape_combine_mask(self.mask, 0, 0) # vllt nach .show()?
        if do_show:
            self.window.show()

    def update_pixmap(self):
        """ update the pixmap """#00006c53ffff
        self.window.window.begin_paint_rect((0, 0, self.size[0], self.size[1]))
        self.window.window.set_back_pixmap(self.pixmap, False)
        self.window.window.end_paint()

BORDER_WIDTH = 0

"""
    I use gnome-OSD (https://launchpad.net/gnome-osd) for the notification now. It's easier for me.

    Thanks to gjc and the Gnome-OSD team!
"""

def do_osd(text, bgcolor, fgcolor, fontdesc,
        use_markup=False, alignment=pango.ALIGN_CENTER,
        fake_translucent_bg=False, drop_shadow=False,
        max_width=None, debug_frame=False, ellipsize=pango.ELLIPSIZE_NONE):
    assert isinstance(fontdesc, pango.FontDescription)
    win = gtk.Window(gtk.WINDOW_POPUP)
    win.add_events(gtk.gdk.ENTER_NOTIFY_MASK)
    darea = gtk.DrawingArea()
    win.add(darea)
    darea.show()
    if use_markup:
        layout = win.create_pango_layout('')
        layout.set_markup(text)
    else:
        layout = win.create_pango_layout(text)
    try:
        layout.set_ellipsize(ellipsize)
    except AttributeError:
        print "ellipsize attribute not supported, ignored"
    layout.set_justify(False)
    layout.set_alignment(alignment)
    layout.set_font_description(fontdesc)
    if max_width is None:
        MAX_WIDTH = gtk.gdk.screen_width() - 8
    else:
        MAX_WIDTH = max_width - 8
    layout.set_width(pango.SCALE*MAX_WIDTH)
    if ellipsize != pango.ELLIPSIZE_NONE:
        layout.set_wrap(pango.WRAP_WORD)
    width, height = layout.get_pixel_size()
    off_x = BORDER_WIDTH*2
    off_y = BORDER_WIDTH*2

    if alignment == pango.ALIGN_CENTER:
        off_x -= MAX_WIDTH/2 - width/2
    elif alignment == pango.ALIGN_RIGHT:
        off_x -= MAX_WIDTH - width
    width += BORDER_WIDTH*4
    height += BORDER_WIDTH*4
    if drop_shadow:
        drop_shadow_distance = max(2, int(fontdesc.get_size()/pango.SCALE*0.1))
        width += drop_shadow_distance
        height += drop_shadow_distance
    darea.set_size_request(width, height)
    darea.realize()
    pixmap = gtk.gdk.Pixmap(darea.window, width, height)
    #pixmap.set_colormap(darea.window.get_colormap())
    fg_gc = gtk.gdk.GC(pixmap); fg_gc.copy(darea.style.fg_gc[gtk.STATE_NORMAL])
    bg_gc = gtk.gdk.GC(pixmap); bg_gc.copy(darea.style.fg_gc[gtk.STATE_NORMAL])
    fg_gc.set_foreground(darea.get_colormap().alloc_color(fgcolor))
    bg_gc.set_background(darea.get_colormap().alloc_color(bgcolor))
    pixmap.draw_rectangle(bg_gc, True, 0, 0, width, height)
    pixmap.draw_layout(fg_gc, off_x, off_y, layout)

    if debug_frame:
        pixmap.draw_rectangle(fg_gc, False, 0, 0, width - 1, height - 1)

    bitmap = gtk.gdk.Pixmap(darea.window, width, height, 1)
    #bitmap.set_colormap(darea.window.get_colormap())
    fg_gc = gtk.gdk.GC(bitmap)
    bg_gc = gtk.gdk.GC(bitmap)
    fg_gc.set_foreground(gtk.gdk.Color(pixel=-1))
    bg_gc.set_background(gtk.gdk.Color(pixel=0))

    if fake_translucent_bg:
        w, h = 2, 2
        stipple = gtk.gdk.Pixmap(None, w, h, 1)
        stipple.draw_rectangle(bg_gc, True, 0, 0, w, h)
        stipple.draw_point(fg_gc, 0, 0)
        stipple.draw_point(fg_gc, 1, 1)
        fg_gc.set_stipple(stipple)
        bitmap.draw_rectangle(bg_gc, True, 0, 0, width, height)
        fg_gc.set_fill(gtk.gdk.STIPPLED)
        bitmap.draw_rectangle(fg_gc, True, 0, 0, width, height)
        fg_gc.set_fill(gtk.gdk.SOLID)

        # draw corners
        corner_size = int(fontdesc.get_size()/pango.SCALE*0.5)
        fg_gc.set_function(gtk.gdk.AND)
        gc = gtk.gdk.GC(bitmap)
        corner = gtk.gdk.Pixmap(bitmap, corner_size, corner_size)
        def draw_corner(angle1, x, y, arc_x0, arc_y0):
            gc.set_foreground(gtk.gdk.Color(pixel=0))
            corner.draw_rectangle(gc, True, 0, 0, corner_size, corner_size)
            gc.set_foreground(gtk.gdk.Color(pixel=1))
            corner.draw_arc(gc, True, arc_x0, arc_y0, corner_size*2, corner_size*2,
                            angle1*64, 90*64)
            bitmap.draw_drawable(fg_gc, corner, 0, 0, x, y, corner_size, corner_size)
        # top-left
        draw_corner(90, 0, 0, 0, 0)
        # bottom-left
        draw_corner(180, 0, height - corner_size, 0, -corner_size)
        # bottom-right
        draw_corner(270, width - corner_size, height - corner_size, -corner_size, -corner_size)
        # top-right
        draw_corner(0, width - corner_size, 0, -corner_size, 0)
        fg_gc.set_function(gtk.gdk.SET)
    else:
        bitmap.draw_rectangle(bg_gc, True, 0, 0, width, height)
    bitmap.draw_layout(fg_gc, off_x, off_y, layout)
    bitmap.draw_layout(fg_gc, off_x + BORDER_WIDTH, off_y, layout)
    bitmap.draw_layout(fg_gc, off_x + BORDER_WIDTH, off_y + BORDER_WIDTH, layout)
    bitmap.draw_layout(fg_gc, off_x, off_y + BORDER_WIDTH, layout)
    bitmap.draw_layout(fg_gc, off_x - BORDER_WIDTH, off_y + BORDER_WIDTH, layout)
    bitmap.draw_layout(fg_gc, off_x - BORDER_WIDTH, off_y, layout)
    bitmap.draw_layout(fg_gc, off_x - BORDER_WIDTH, off_y - BORDER_WIDTH, layout)
    bitmap.draw_layout(fg_gc, off_x, off_y - BORDER_WIDTH, layout)
    bitmap.draw_layout(fg_gc, off_x + BORDER_WIDTH, off_y - BORDER_WIDTH, layout)

    if drop_shadow:
        bitmap.draw_layout(fg_gc, off_x + drop_shadow_distance,
                           off_y + drop_shadow_distance, layout)
    if debug_frame:
        bitmap.draw_rectangle(fg_gc, False, 0, 0, width - 1, height - 1)

    darea.window.set_back_pixmap(pixmap, False)
    win.window.shape_combine_mask(bitmap, 0, 0)
    win.width = width
    win.height = height
    return win

class OSDNotifier():
    def __init__(self, color='red', f="Sans", fs=20):
        self.color = color
        self.font = f
        self.font_size = int(fs)
        self.windows = []

    def notify(self, s, g):
        w = do_osd(s, self.color, self.color, pango.FontDescription("%s %d" % (self.font, self.font_size)))
        w.move(10, gtk.gdk.screen_height() - w.height - 10)
        w.show()
        self.windows.append(w)

    def destroy(self):
        if self.windows:
            window = self.windows[0]
            window.hide()
            window.destroy()
            window = None
            del self.windows[0]

    def clear(self):
        self.destroy()
