# -*- coding: utf-8 -*-
"""
    GESTIKK
    Version 0.6.1

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007-2008:
    Friedrich 'fred.reichbier' Weber, Jonas 'Dauerbaustelle' Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""

import gestikk.dry as dry

import sys

import os
import signal
import subprocess
# GTK
if dry.try_to_import("pygtk", "Please install gtk2 python bindings!", True):
    import pygtk
    pygtk.require("2.0")
    import gtk

    if not dry.check_gtk_version(gtk.ver, (2, 12, 0)):
        print 'ERROR: You need pygtk >= 2.12.0 to run gestikk!'
        sys.exit(1)

if dry.try_to_import("gobject", "Please install gobject python bindings!", True):
    # Gobject
    import gobject

if dry.try_to_import("gtk.glade", "Please install glade2 python bindings!", True):
    # Glade
    import gtk.glade

if dry.try_to_import("kiwi", "Please install kiwi for python!", True):
    # Kiwi
    import kiwi
    import kiwi.ui.widgets.combobox, kiwi.ui.views, kiwi.ui.objectlist
    import kiwi.ui.dialogs
    kiwiColumn = kiwi.ui.objectlist.Column

from gestikk.tools import *

class _ModifierObject():
    def __init__(self, gnome='', human=''):
        self.gnome = gnome # gnome notation
        self.human = human # human - readable
        self.active = False # active?

    def reset(self):
        self.active = False


MODS = [_ModifierObject('<Control>', 'Control'), _ModifierObject('<Alt>', 'Alt'), _ModifierObject('<Shift>', 'Shift'), _ModifierObject('<Super>', 'Super')]

class KeyStrokeAssistant(gtk.Dialog):
    """ a dialog where users can click their keystrokes """
    def __init__(self, title='', parent=None):
        gtk.Dialog.__init__(self, 'Keystroke', parent, 0, (gtk.STOCK_OK, gtk.RESPONSE_OK, \
                                                          gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        self.keystroke = ''
        columns = [kiwiColumn('human'), kiwiColumn('active', data_type=bool, editable=True)]
        self.contentbox = gtk.HBox(0, 3)
        self.contentbox.show()
        self.vbox.pack_start(self.contentbox, True, True)
        self.vbox2 = gtk.VBox(0, 3)
        self.vbox2.show()
        self.infobutton = gtk.Button(None, gtk.STOCK_ABOUT)
        self.infobutton.show()
        self.infobutton.connect("clicked", self.infobutton_clicked)

        mods = kiwi.ui.objectlist.ObjectList(columns)
        mods.show()
        mods.set_size_request(150, 150)
        self.contentbox.pack_start(mods, True, True)
        self.mods = mods

        self.contentbox.pack_start(self.vbox2, True, True)

        key = KeyRecognizer()
        key.show()
        self.vbox2.pack_start(key, True, True)
        self.key = key

        self.vbox2.pack_start(self.infobutton, False, False)

        self.mods.add_list(MODS)
        self.connect('response', self.sig_response)

    def infobutton_clicked(self, evt):
        def evt_close_info(self, window):
            window.destroy()
        wTree = gtk.glade.XML("gestikk.glade") #self.gladefile
        infowindow = wTree.get_widget("infodialog")
        infotitel = wTree.get_widget("info_titel")
        infotext = wTree.get_widget("info_text")
        infotitel.set_label("<b><span size=\"xx-large\">%s</span></b>" % _("Keystroke assistant"))
        infotext.set_text("With this assistant, you can choose modifiers and keys in a simple, intuitive way. Just select the modifers on the left and press the key in the box on the right.")
        infowindow.show()
        wTree.get_widget('info_ok').connect('clicked', evt_close_info, infowindow)


    def sig_response(self, dialog, response_id):
        gmod = [] # list of gnome compatible modifier keys
        if response_id == gtk.RESPONSE_OK:
            for mod in MODS:
                if mod.active:
                    gmod.append(mod.gnome)
                mod.reset()
        gmod_str = ''.join(gmod)
        key = self.key.get_text()
        self.keystroke = gmod_str + key
        self.hide()


class KeyRecognizer(gtk.Entry):
    """ simple widget for recognizing shortcuts """
    def __init__(self):
        gtk.Entry.__init__(self)
        self.set_property('editable', False)
        self.connect('key-press-event', self.sig_keypress)

    def sig_keypress(self, w, event):
        """ signal: key press event! """
        if event.state & gtk.gdk.MOD2_MASK == gtk.gdk.MOD2_MASK:
            # remove numlock
            event.state ^= gtk.gdk.MOD2_MASK
        if event.state & gtk.gdk.MOD4_MASK == gtk.gdk.MOD4_MASK \
           and event.state & gtk.gdk.SUPER_MASK == gtk.gdk.SUPER_MASK:
            event.state ^= gtk.gdk.MOD4_MASK # only SUPER, not MOD4
        self.set_text(gtk.accelerator_name(event.keyval, event.state))

class ConditionDialog():
    """ condition dialog """
    def __init__(self, logic, action, configwindow, add_gesture=True, run=True, callback=None):
        """
            :param logic: The Logic()
            :param action: The Action(), should be ready except the condition
        """
        global _
        _ = init_gettext().gettext # i _really_ hate such workarounds. But they are so useful.
        self.add_gesture = add_gesture
        self.clicked_ok = True
        self.callback = callback
        self.configwindow = configwindow
        self.logic = logic
        self.action = action
        self.wtree = self.logic._configuration_window.new_wtree()

        self.dialog = self.wtree.get_widget('dialog-condition')
        self.v_condition = self.wtree.get_widget('condition-condition')
        self.v_refresh = self.wtree.get_widget('condition-btn-refresh')
        self.v_refresh.connect('clicked', self.evt_refresh)
        self.v_test = self.wtree.get_widget('condition-btn-test')
        self.v_test.connect('clicked', self.evt_test)

        self.v_list = kiwi.ui.objectlist.ObjectList([kiwiColumn('key', title=_('Function'), sorted=True),\
                                                     kiwiColumn('value', title=_('Value'))])
        self.wtree.get_widget('condition-vbox').pack_start(self.v_list, True, True)
        self.wtree.get_widget('condition-vbox').reorder_child(self.wtree.get_widget('hbox3'), 3)
        self.v_list.show()

        self.setup()
        if not self.logic.enable_conditions:
            self.wtree.get_widget('condition-vbox').set_sensitive(False)
        if run:
            self.dialog.run()

    def _test(self):
        """ Test the given condition """
        self.show_it()
        condition = self.v_condition.get_text()
        result = condition_handler.check(condition)
        s = {False: _('The condition is not fulfilled!'), True: _('The condition is fulfilled!')}.get(result)
        kiwi.ui.dialogs.messagedialog(gtk.MESSAGE_INFO, s, '', self.dialog, gtk.BUTTONS_OK)

    def _refresh(self):
        """ get all values """
        self.show_it()
        # get values
        objs = dict_to_objects(wm_handler.get_all())
        self.v_list.add_list(objs)

    def show_it(self):
        """ show everything ... """
        self.configwindow.get_widget('configdialog').show()
        self.dialog.show()

    def hide_it(self):
        """ say that we want to hide gestikk - and do it """
        #
        kiwi.ui.dialogs.messagedialog(gtk.MESSAGE_INFO, _('Hiding gestikk'), \
        _("gestikk will now be hidden.\nIt'll be back and refresh in 5 seconds."), \
        self.dialog, gtk.BUTTONS_OK)
        #
        self.configwindow.get_widget('configdialog').hide()
        self.dialog.hide()

    def evt_refresh(self, evt):
        """ refresh all values - signal handler """
        self.hide_it()
        gobject.timeout_add(5000, self._refresh)

    def dry_test(self):
        """ do a little dry test.
            :returns: True if syntax is correct, False if it isn't
        """
        try:
            condition_handler.check(self.v_condition.get_text())
        except (condparse.ParseException, condparse.pyp.ParseException), e:
            kiwi.ui.dialogs.messagedialog(gtk.MESSAGE_ERROR, _('A parsing error occured!'), str(e), self.dialog, gtk.BUTTONS_OK)
            return False
        else:
            return True

    def evt_test(self, evt):
        """ test the condition """
        # first: 'dry' test, display exceptions
        res = self.dry_test()
        if not res:
            return
        # all correct: test it
        self.hide_it()
        gobject.timeout_add(5000, self._test)

    def setup(self):
        """ reset all values """
        # set signal handlers
        # first disconnect all
        self.dialog.connect("response", self.evt_response)
        # reset values
        self.v_condition.set_text('')

    def evt_response(self, dialog, response_id):
        """ dialog response """
        if response_id == gtk.RESPONSE_OK:
            # first: test its syntax. we don't want wrong conditions
            res = self.dry_test()
            if not res:
                return
            # OK, add it
            # unhide conf window
            self.action.condition = self.v_condition.get_text()
            if self.add_gesture:
                self.logic.add_gesture(self.action)
                self.logic._configuration_window.reload_gestures()
            self.clicked_ok = True
            if self.callback:
                self.callback(self)
        else:
            self.clicked_ok = False
        self.dialog.hide()
        self.dialog.destroy()
        self.configwindow.get_widget("configdialog").set_sensitive(True)

class ActionKeyDialog():
    """ the dialog in which we define the keystroke of the key action """
    def __init__(self, logic, action, configwindow):
        """
            :param logic: The Logic()
            :param action: The Action()
        """
        self.configwindow = configwindow
        self.logic = logic
        self.action = action
        self.wtree = self.logic._configuration_window.new_wtree()

        self.dialog = self.wtree.get_widget('dialog-action-key')
        self.v_key = gtk.Entry()
        self.v_key.show()
        self.get_widget("table5").attach(self.v_key, 1, 2, 0, 1, \
                                    gtk.EXPAND | gtk.FILL, gtk.EXPAND)
        self.setup()

        self.dialog.run()

    def setup(self):
        """ reset all values """
        # set signal handlers
        # first disconnect all
        self.dialog.connect("response", self.evt_response)
        self.wtree.signal_connect("evt_act_key_btntest", self.evt_test)
        self.wtree.signal_connect("evt_act_key_assistant", self.evt_assistant)
        # reset values
        self.v_key.set_text('')

    def get_widget(self, name):
        return self.wtree.get_widget(name)

    def evt_test(self, evt):
        """ test the keystroke """
        simulate_keys(keystroke_to_x11(self.v_key.get_text()))

    def evt_assistant(self, evt):
        """ launch assistant """
        ka = KeyStrokeAssistant()
        ka.run()
        if ka.keystroke:
            self.v_key.set_text(ka.keystroke)
        ka.destroy()

    def evt_response(self, dialog, response_id):
        """ dialog response """
        self.dialog.hide()
        self.dialog.destroy()
        if response_id == gtk.RESPONSE_OK:
            self.action.parameter = self.v_key.get_text()
            # show the condition dialog
            dialog = ConditionDialog(self.logic, self.action, self.configwindow)
        else:
            self.configwindow.get_widget("configdialog").set_sensitive(True)

class ActionCommandDialog():
    """ the dialog in which we define the command of the command action """
    def __init__(self, logic, action, configwindow):
        """
            :param logic: The Logic()
            :param action: The Action()
        """
        self.configwindow = configwindow
        self.logic = logic
        self.action = action
        self.wtree = self.logic._configuration_window.new_wtree()
        self.dialog = self.wtree.get_widget('dialog-action-cmd')
        self.dialog.connect("response", self.evt_response)
        self.v_cmd = self.wtree.get_widget('act-cmd-command')

        self.wtree.signal_connect("evt_act_cmd_btntest", self.evt_test)
        self.setup()

        self.dialog.run()

    def setup(self):
        """ reset all values """
        # reset values
        self.v_cmd.set_text('')

    def get_widget(self, name):
        return self.wtree.get_widget(name)

    def evt_test(self, evt):
        """ test the command """
        execute(self.v_cmd.get_text())

    def evt_response(self, dialog, response_id):
        """ dialog response """
        self.dialog.hide()
        self.dialog.destroy()
        if response_id == gtk.RESPONSE_OK:
            self.action.parameter = self.v_cmd.get_text()
            # show the specified configuration dialog
            dialog = ConditionDialog(self.logic, self.action, self.configwindow)
        else:
            self.configwindow.get_widget("configdialog").set_sensitive(True)


class ActionDialog():
    """ the dialog in which we define the action """
    def __init__(self, logic, gesture, configwindow):
        """
            :param logic: The Logic()
            :param gesture: e.g. '[(1,0),(-1,1)]'
            :type gesture: String
        """
        self.configwindow = configwindow
        self.logic = logic
        self.gesture = gesture
        self.wtree = self.logic._configuration_window.new_wtree()
        self.dialog = self.wtree.get_widget('actiondialog')
        self.dialog.connect("response", self.evt_response)
        self.setup()

        self.dialog.run()

    def setup(self):
        """ set-up the kiwi boxes. this function deletes itself after
        execution. that method is not very clean, but it works. TODO. """
        # make kiwi's
        v_aswitch = kiwi.ui.widgets.combobox.ComboBox()
        v_aswitch.data_type = str
        v_aswitch.show()
        v_aswitch.prefill(self.logic.action_list.keys(), sort=True)
        self.v_aswitch = v_aswitch
        self.get_widget("vbox3").pack_start(v_aswitch, True, False)
        # delete
        #ActionDialog._kiwi_done = True
        # reset values

    def get_widget(self, name):
        return self.wtree.get_widget(name)

    def evt_response(self, dialog, response_id):
        self.dialog.hide()
        self.dialog.destroy()
        """ dialog response """
        if response_id == gtk.RESPONSE_APPLY:
            # new Action_...()
            template = self.logic.action_list[self.v_aswitch.get_selected()]
            action = template.caction(self.gesture[1:-1], '', '') # create an action
            # show the specified configuration dialog
            if template.caction == Action_Command: # command?
                dialog = ActionCommandDialog(self.logic, action, self.configwindow)
            elif template.caction == Action_Key:
                dialog = ActionKeyDialog(self.logic, action, self.configwindow)
        else:
            self.configwindow.get_widget("configdialog").set_sensitive(True)

class GestureRecognitionDialog():
    """ the dialog that recognizes a new gesture """
    def __init__(self, logic, configwindow, xsize=700, ysize=500):
        self.configwindow = configwindow
        # wtf?
        self.un_lock_configwindow(False, False, False)
        self.logic = logic
        self.running = True
        self.gesture_positions = []
        self.gestures = []
        # gtk stuff
        self.root_window = gtk.gdk.get_default_root_window() # todo
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_default_size(xsize, ysize)
        self.window._label = gtk.Label(_('Draw the gesture (big one!) in the window while pressing the right mouse button.'))
        self.window.add(self.window._label)
        self.window._label.show()
        self.window.set_title(_('gestikk'))
        self.window.connect('delete-event', self.un_lock_configwindow)
        # wtf? ueberliefert 4 Parameter(!)
        self.callback = None
        self.run()

    def un_lock_configwindow(self, window=False, evt=False, param=True):
        # wtf? fucking parameters
        # ....but it works -.-
        #print param
        self.configwindow.get_widget("configdialog").set_sensitive(param)

    def run(self):
        """ start """
        self.window.show()
        self.window.set_keep_above(True)
        """ run the thread """
        gobject.timeout_add(100, self._listen)

    def check_gestures(self):
        """ Check both gestures in self.gestures """
        if self.gestures[0] != self.gestures[1]: # not the same
            self.window._label.set_label(_('Unfortunately the gestures don\'t match.\nTry again please!'))
            self.gestures = []
        else:
            gestures = self.gestures[:]
            self.gestures = []
            self.running = False
            self.window.destroy()
            #print gestures
            #self.callback(gestures[0])
            dialog = ActionDialog(self.logic, gestures[0], self.configwindow)

    def _listen(self):
        # get x, y and mouse status (clicked etc.)
        x, y, state = self.root_window.get_pointer()
        # re-arm timer
        if self.running:
            gobject.timeout_add(100, self._listen)
        if state & gtk.gdk.BUTTON3_MASK == gtk.gdk.BUTTON3_MASK:
            # pressing right mouse button
            self.gesture_positions.append((x, y))
        else: # Not longer pressed
            if self.gesture_positions: # gesture finished
                # Pfad errechnen
                gesture = []
                tolerance = int(self.logic.config_get_option('Preferences', 'tolerance', '25'))
                for gid in range(1, len(self.gesture_positions)): # let first out
                    gesture.append(calc_direction(self.gesture_positions[gid-1], \
                                                self.gesture_positions[gid], tolerance))
                #print gesture
                # Remove zeros
                while (0,0) in gesture:
                    gesture.remove((0,0))
                # Delete duplicates
                gesture = remove_duplicates(gesture)
                #print gesture
                self.gesture_positions = []
                # Is a command assigned to the gesture?
                gesture_repr = repr(gesture).replace(' ', '') # w/o spaces
                #print gesture_repr
                self.gestures.append(gesture_repr)
                if gesture_repr == '[]': # empty?
                    self.window._label.set_label(_('That was too fast.\nPlease do it again.'))
                    self.gestures = []
                    self.gesture_positions = []
                    return
                self.gesture_positions = []
                if len(self.gestures) == 1:
                    # first gesture: encourage the user
                    self.window._label.set_label(_('Now again please so that I can see,\nif you misdrawed it :-)'))
                else:
                    # Second gesture finished, check it
                    self.check_gestures()
                if DEBUG:
                    print 'Gesture: %s' % gesture_repr

class ConfigurationWindow():
    """ the main configuration window. gets the interface from gestikk.glade """
    def __init__(self, logic, Xsize=400, Ysize=400):
        """ :param logic: The gestikk_tools.Logic() instance
            :param Xsize, Ysize: desired width and height
        """
        global _
        _ = init_gettext().gettext # i _really_ hate such workarounds. But they are so useful.

        self.changed = False
        self.old_gestures = []

        self.logic = logic
        self.logic._configuration_window = self
        # load the .glade file
        self.gladefile = get_file("gestikk.glade", "gladefile")
        self.wTree = self.new_wtree()
        # import widgets tree from glade file as wTree
        self.kiwi_widgets = {}
        # workaround für widgets, die mit kiwi erstellt werden
        self.connect_all()
        # we want a clean quit
        self.get_widget("configdialog").connect('delete-event', self.evt_btn_close)
        # resize window (widget configdialog)
        self.get_widget("configdialog").resize(Xsize, Ysize)
        # set icon
        self.get_widget("configdialog").set_icon_from_file(get_file("gestikk.xpm", "icon"))
        # --- generate kiwi widgets
        # -- gesture switchers combobox
        v_gswitch = kiwi.ui.widgets.combobox.ComboBox()
        v_gswitch.connect("changed", self.evt_changed)
        v_gswitch.data_type = str
        v_gswitch.show()
        self.kiwi_widgets["v_gswitch"] = v_gswitch
        self.get_widget("table1").attach(v_gswitch, 1, 2, 1, 2, \
                                    gtk.EXPAND | gtk.FILL, gtk.EXPAND)
        # -- action notification combobox
        v_notify = kiwi.ui.widgets.combobox.ComboBox()
        v_notify.connect("changed", self.evt_changed)
        v_notify.connect("changed", self.evt_notify_toggle)
        v_notify.data_type = str
        v_notify.show()
        self.kiwi_widgets["v_notify"] = v_notify
        self.get_widget("notify_hbox1").pack_start(v_notify, True, True, False)
        # -- gestures list
        v_gestures_columns = \
                        [kiwiColumn("gesture", title=_('Gesture'), sorted=True),
                         kiwiColumn("condition", title=_('Condition')),
                         kiwiColumn("action_formatted", title=_('Action'))]
        v_gestures = kiwi.ui.objectlist.ObjectList(v_gestures_columns)
        self.kiwi_widgets["v_gestures"] = v_gestures
        v_gestures.show()
        self.get_widget("vbox1").pack_start(v_gestures, True, True)
        self.get_widget("vbox1").reorder_child(self.get_widget("hbox1"), 1)
        self.get_widget("osd_frame").set_sensitive(False) # TODO
        self.get_widget("configdialog").show()

        self.get_widget('v_globalcond').connect('key-press-event', \
                                lambda spam, eggs: not eggs.keyval in [0xff51, 0xff53, 0xff57, 0xff50])
        # make it read - only

        # defines info texts for evt_showinfo(), starting with infobutton00
        self.infotext = [i.replace('\n', '') for i in [_("Defines the threshold value. A higher value means higher error-tolerance."), \
                '', \
        _("Defines the mouse or keyboard\nbutton which has to be pressed\nwhen executing a gesture."), \
        _("Defines the color used to draw\nthe gesture on the screen."), \
        _("Defines the line width of the gesture\ndrawed on the screen."), \
        _("Defines the font used\nfor OSD notification."), \
        _("Defines the color used\nfor OSD notification.")]] # TODO: This is not nice. really. As we have text wrapping now, I have to strip all \n. Make it better in gestikk 0.7 ;-)
        # defines info text titels
        self.infotext_title = [_("Tolerance"), "", _("Gesture switch"), _("Color"),\
        _("Line Width"), _("OSD font"), _("OSD color")]
        # load the configuration
        self.load_configuration()

    def quitdialog(self):
        """ dialog which asks for saving the options """
        if self.quitdialog_ask_again("get") == '0':
            self.get_widget("quitdialog").show_all()
        else:
            gtk.main_quit()

    def quitdialog_ask_again(self, func):
        if func == "get":
            return self.dont_ask_again
        elif func == "set":
            if self.get_widget("dont_ask_again").get_active() == True:
                self.logic.config_set_option('Preferences', 'dont_ask_again', '1')
                self.logic.config_write()
                self.logic.reload_config()

    def evt_quitdialog_buttonno(self,evt):
        self.quitdialog_ask_again("set")
        gtk.main_quit()

    def evt_quitdialog_buttonyes(self,evt):
        self.quitdialog_ask_again("set")
        self.evt_btn_save(self)

    def reload_gestures(self):
        """ reload the gesture list """
        # set the gestures
        self.get_widget("v_gestures").add_list(dict_unzip(self.logic.gestures))

    def new_wtree(self):
        """ return a new gtk.glade.XML """
        return gtk.glade.XML(self.gladefile)

    def get_widget(self, name):
        """ get a widget by its name - wrapper for glade.XML and kiwi_widgets
            :param name: The name
        """
        if name not in self.kiwi_widgets:
            return self.wTree.get_widget(name)
        else:
            return self.kiwi_widgets[name]

    def connect_all(self):
        """ connect functions from glade into Python defs """
        dic = { "evt_new_gesture" : self.evt_new_gesture,
                "evt_view_gesture" : self.evt_view_gesture,
                "evt_delete_gesture" : self.evt_delete_gesture,
                "evt_btn_close" : self.evt_btn_close,
                "evt_btn_save" : self.evt_btn_save,
                "evt_osd_toggle" : self.evt_osd_toggle,
                "evt_quitdialog_buttonno": self.evt_quitdialog_buttonno,
                "evt_quitdialog_buttonyes": self.evt_quitdialog_buttonyes,
                "evt_changed": self.evt_changed,
                "evt_showinfo": self.evt_showinfo,
                "evt_condedit": self.evt_condedit
              }
        self.wTree.signal_autoconnect(dic)

    def options_changed(self):
        """ return whether options were changed. check self.changed and the gestures """
        if (self.changed or self.old_gestures != self.logic.gestures):
            return True
        else:
            return False

    def evt_condedit(self, evt):
        """ signal: user wants to set a global condition """
        def refresh_cond(c):
            if c.clicked_ok:
                self.get_widget('v_globalcond').set_text(c.action.condition)
        action = Action('', self.get_widget('v_globalcond').get_text(), '')
        cd = ConditionDialog(self.logic, action, self, False, False, refresh_cond) # TODO. has to be much nicer.
        cd.v_condition.set_text(action.condition)
        cd.dialog.run()

    def evt_showinfo(self, evt):
        def evt_close_info(self, window):
            window.destroy()
        # read out the last two chars of button name and save them with int() as number
        num = int(str(evt.get_name()[len(evt.get_name())-2])+\
        str(evt.get_name()[len(evt.get_name())-1]))
        wTree = self.new_wtree()
        infowindow = wTree.get_widget("infodialog")
        infowindow.set_transient_for(self.get_widget("configdialog"))
        infotitel = wTree.get_widget("info_titel")
        infotext = wTree.get_widget("info_text")
        infotitel.set_label("<b><span size=\"xx-large\">"+self.infotext_title[num]+"</span></b>")
        infotext.set_text(self.infotext[num])
        infowindow.show()
        wTree.get_widget('info_ok').connect('clicked', evt_close_info, infowindow)

    def evt_changed(self, evt):
        """ something changed """
        self.changed = True

    def evt_notify_toggle(self, evt):
        if dict_rev(self.logic.notifiers)[self.get_widget("v_notify").get_selected()] == "osd":
            self.get_widget("notify_frame").set_sensitive(True)
        else:
            self.get_widget("notify_frame").set_sensitive(False)

    def evt_osd_toggle(self, evt):
        if self.get_widget("osd_activate").get_active() == True:
            self.get_widget("osd_frame").set_sensitive(True)
        else:
            self.get_widget("osd_frame").set_sensitive(False)

    def get_option(self, section, option, default=''):
        """ Wrapper for self.logic.config_get_option """
        return self.logic.config_get_option(section, option, default)

    def evt_new_gesture(self, evt):
        """ Event / Signal: create new gesture """
        # first: recognize the gesture
        dialog = GestureRecognitionDialog(self.logic, self)

    def evt_view_gesture(self, evt):
        """ Event / Signal: view current gesture """
        sel = self.get_widget("v_gestures").get_selected()
        if sel:
            vwd = ViewGestureDialog(gesture2py(sel.gesture))
            vwd.show()

    def evt_delete_gesture(self, evt):
        """ Event / Signal: delete a gesture """
        sel = self.get_widget("v_gestures").get_selected()
        if sel:
            self.logic.gestures[sel.gesture].remove(sel)
            self.reload_gestures()

    def evt_btn_close(self, evt, that_is_not_very_interesting=None):
        """ Event / Signal: abort / close """
        if self.options_changed():
            self.quitdialog()
        else:
            gtk.main_quit()

    def evt_btn_save(self, foobar):
        self.save_configuration()
        gtk.main_quit()

    def get_current_vgswitch(self):
        return self.logic.gesture_switchers[self.logic.gesture_switchers_txt[self.get_widget("v_gswitch").get_selected()]]

    def load_configuration(self):
        """ gets the configuration """
        # Konfiguration hier einlesen
        v_tolerance = self.get_option('Preferences', 'tolerance', '20')
        osd_enable = int(self.get_option('OSD', 'enabled', '0'))
        v_osd_line_width = int(self.get_option('OSD', 'line_width', '3'))
        self.dont_ask_again = self.get_option('Preferences', 'dont_ask_again', '0')
        colorstr = self.get_option('OSD', 'color', '#ff0000')
        notify_colorstr = self.get_option('Notification', 'color', '#ff0000')
        notify_font = self.get_option('Notification', 'font', 'Sans')
        notify_fontsize = self.get_option('Notification' , 'font_size', '20')
        globalcond = self.get_option('Preferences', 'global condition', '')

        # set global condition
        self.get_widget('v_globalcond').set_text(globalcond)
        # set Gesture switchers
        self.get_widget("v_gswitch").prefill(self.logic.gesture_switchers_txt.keys(), sort=True)
        self.get_widget("v_gswitch").select(self.logic.current_switcher_txt)
        # set Notifier
        self.get_widget("v_notify").prefill(self.logic.notifiers.values())
        self.get_widget("v_notify").select(self.logic.notifiers[self.logic.config_get_option("Notification", "type", "0")])
        # set notify color
        self.get_widget("notify_color_button").set_color(gtk.gdk.color_parse(notify_colorstr))
        # set notify font
        self.get_widget("notify_font_button").set_font_name("%s %s" % (notify_font, notify_fontsize))
        # set the other values
        self.get_widget("v_tolerance").set_text(v_tolerance)
        self.get_widget("v_osd_line_width").set_value(v_osd_line_width)
        self.get_widget("osd_color_button").set_color(gtk.gdk.color_parse(colorstr))
        # reload the gestures
        self.reload_gestures()

        # set osd_frame sensitive
        if not osd_enable:
            self.get_widget("osd_frame").set_sensitive(False)
        else:
            self.get_widget("osd_frame").set_sensitive(True)
            self.get_widget("osd_activate").set_active(True)

        # because we set the values, it caused the signal "changed". now we reset it.
        self.changed = False
        # save the gestures
        self.old_gestures = self.logic.gestures.copy()

    def save_configuration(self):
        """ saves the configuration """
        # get
        v_tolerance = self.get_widget("v_tolerance").get_text() # get tolerance
        # get osd_enable checkbox
        osd_enable = self.get_widget("osd_activate").get_active()

        # set
        # Preferences
        self.logic.config_set_option('Preferences', 'tolerance', v_tolerance)
        self.logic.config_set_option('Preferences', 'switcher', self.get_current_vgswitch())
        self.logic.config_set_option('Preferences', 'global condition', self.get_widget('v_globalcond').get_text())
        # On Screen Gesture Display
        self.logic.config_set_option('OSD', 'line_width', int(self.get_widget("v_osd_line_width").get_value()))
        # osd_enable option
        self.logic.config_set_option('OSD', 'enabled', str(int(osd_enable)))
        # color button
        self.logic.config_set_option('OSD', 'color', self.get_widget("osd_color_button").get_color().to_string())
        # Notifications
        self.logic.config_set_option('Notification', 'type', dict_rev(self.logic.notifiers)[self.get_widget("v_notify").get_selected()])
        self.logic.config_set_option('Notification', 'color', self.get_widget("notify_color_button").get_color().to_string())
        #save notify font
        font=self.get_widget("notify_font_button").get_font_name().split(" ")
        self.logic.config_set_option('Notification', 'font', font[0])
        self.logic.config_set_option('Notification', 'font_size', font[len(font)-1])
        # save the gestures
        self.logic.save_gestures()

        # save
        self.logic.config_write()
        # send signal to the gestikk main process (the parent process)
        ppid = os.getppid()
        if is_gestikk(ppid):
            os.kill(ppid, signal.SIGUSR1)

def main(debug):
    """ main. This is called by the gestikk.py script. """
    global DEBUG
    DEBUG = debug
    logic = Logic()
    logic.enable_conditions = CONDITION_SUPPORT
    if logic.config_get_option('Preferences', 'condition support', 'yes') != 'yes': # undocumented ini option ^_^
        print 'Disabling conditions. You set this in the INI file.'
        logic.enable_conditions = False
    get_rid_of_this_variable = ConfigurationWindow(logic)
    gtk.main()

if __name__=="__main__":
    main(False)
